from practicum import find_mcu_boards, McuBoard, PeriBoard
from flask import Flask, render_template, redirect, url_for,request
from g3 import g3sensor
from RPi import GPIO
import sys
print(sys.path)
sys.path.append('/home/james/.local/lib/python2.7/site-packages')
from flask_cors import CORS
from datetime import datetime
import drivers

air=g3sensor()
devices = find_mcu_boards()
mcu = McuBoard(devices[0])
peri = PeriBoard(mcu)
GPIO.setmode(GPIO.BCM)
GPIO.setwarnings(False)
pins = [17,5,22]
# fan : 17
# green : 4
# red : 27
GPIO.setup(pins, GPIO.OUT, initial = GPIO.HIGH)
work = False
date_base = datetime.now()
timeset = 0

app = Flask(__name__)
cors = CORS(app, resource={r"/": {"origins": "*"}})

@app.route("/")
def index():
   if peri.get_switch():
       sw = "Pressed"
   else:
       sw = "Released"
   light = peri.get_light()
   try:
       pmdata=air.read("/dev/ttyS0")
   except:
       pmdata=[0,0,0,0,0,0]
   print (pmdata)
   return """
     <h1>Air purifier</h1>
     <div>
         Air Quality
     <ul>
         <li>PM1 : {} </li>
         <li>PM2.5 : {}</li>  
         <li>PM10 : {} </li>
     </ul>
     </div>
     <div>
       LED control:
       <ul>
         <li>Red: <a href="/led/0/0">OFF</a> | <a href="/led/0/1">ON</a></li>
         <li>Yellow: <a href="/led/1/0">OFF</a> | <a href="/led/1/1">ON</a></li>
         <li>Green: <a href="/led/2/0">OFF</a> | <a href="/led/2/1">ON</a></li>
       </ul>
     </div>
   """.format(pmdata[3],pmdata[5],pmdata[4])

@app.route("/readdata", methods=['GET'])
def read_data():
    global work
    t = datetime.now()
    sec = (datetime.now()-date_base).total_seconds()
    if sec >= timeset:
        work = False
    print (work,sec,':',timeset)
    pm = air.set_status(pins,work)
    print(pm)
    return {'result' : pm} 

@app.route("/setfan", methods = ["POST"])
def setfan():
    global work 
    global date_base
    global timeset
    data = request.json
    print(data)
    work = True
    date_base = datetime.now()
    timeset = int( data['minute']*60 + data['second'])
    print(work)
    return {'result': timeset}

@app.route("/led/<int:num>/<int:state>")
def set_led(num,state):
   peri.set_led(num,state)
   return redirect(url_for("index"))

@app.route("/stopfan", methods = ["GET"])
def stop():
    global work
    work = 0
    return {'result' : 'stop success'}

if __name__ == '__main__':
    try:
        app.run(debug=True, port=8000, host="0.0.0.0")
    except KeyboardInterrupt:
    # If there is a KeyboardInterrupt (when you press ctrl+c), exit the program and clean$
        print("Cleaning up!")
        GPIO.cleanup()
        display.lcd_clear()
